# This file was auto-generated by Fern from our API Definition.

from ..core.pydantic_utilities import UniversalBaseModel
import typing
from .add_openai_api_key_enum import AddOpenaiApiKeyEnum
from ..core.pydantic_utilities import IS_PYDANTIC_V2
import pydantic


class OpenAiVectorizerConfig(UniversalBaseModel):
    """
    Configuration for using an OpenAI vectorizer.
    """

    add_openai_api_key: typing.Optional[AddOpenaiApiKeyEnum] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow
