# This file was auto-generated by Fern from our API Definition.

import typing
from .test_case_string_variable_value import TestCaseStringVariableValue
from .test_case_number_variable_value import TestCaseNumberVariableValue
from .test_case_json_variable_value import TestCaseJsonVariableValue
from .test_case_chat_history_variable_value import TestCaseChatHistoryVariableValue
from .test_case_search_results_variable_value import TestCaseSearchResultsVariableValue
from .test_case_error_variable_value import TestCaseErrorVariableValue
from .test_case_function_call_variable_value import TestCaseFunctionCallVariableValue
from .test_case_array_variable_value import TestCaseArrayVariableValue

TestCaseVariableValue = typing.Union[
    TestCaseStringVariableValue,
    TestCaseNumberVariableValue,
    TestCaseJsonVariableValue,
    TestCaseChatHistoryVariableValue,
    TestCaseSearchResultsVariableValue,
    TestCaseErrorVariableValue,
    TestCaseFunctionCallVariableValue,
    TestCaseArrayVariableValue,
]
