# This file was auto-generated by Fern from our API Definition.

import typing
from .string_vellum_value import StringVellumValue
from .json_vellum_value import JsonVellumValue
from .error_vellum_value import ErrorVellumValue
from .function_call_vellum_value import FunctionCallVellumValue

PromptOutput = typing.Union[StringVellumValue, JsonVellumValue, ErrorVellumValue, FunctionCallVellumValue]
