# This file was auto-generated by Fern from our API Definition.

import typing

VellumSdkErrorCodeEnum = typing.Union[
    typing.Literal[
        "INVALID_WORKFLOW",
        "INVALID_INPUTS",
        "INVALID_OUTPUTS",
        "INVALID_STATE",
        "INVALID_CODE",
        "INVALID_TEMPLATE",
        "INTERNAL_ERROR",
        "PROVIDER_CREDENTIALS_UNAVAILABLE",
        "PROVIDER_ERROR",
        "USER_DEFINED_ERROR",
        "WORKFLOW_CANCELLED",
        "NODE_EXECUTION",
    ],
    typing.Any,
]
