# This file was auto-generated by Fern from our API Definition.

from ..core.pydantic_utilities import UniversalBaseModel
import typing
from .code_resource_definition import CodeResourceDefinition
from ..core.pydantic_utilities import IS_PYDANTIC_V2
import pydantic


class ExternalInputDescriptor(UniversalBaseModel):
    types: typing.List[CodeResourceDefinition]
    inputs_class: typing.Optional[CodeResourceDefinition] = None
    instance: typing.Optional[typing.Optional[typing.Any]] = None
    name: str

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow
